#!/bin/sh
#
# powerplot-rti -- Generate Total power RTI plots  
# for Scion HF radar raw data files :  Shuffle Mode
# Terry Bullett
# University of Colorado
#
# Version 2.05  29Sep10
# useage:
# powerplot-rti.sh {switches} RIQFILES
#
# Requires:
#    msttcorefonts (included)
#    bc
#    riq-ionogram version 2.04+
#    station_settings.sh
# 
# Version 2.05 -- Moved station-based settings into a separate (sourced) function.
#
export GDFONTPATH=/usr/share/fonts/truetype/msttcorefonts/
#export GDFONTPATH=$HOME/lib/msttcorefonts/

APPNAME=powerplot-rti
TFMT='%F %T %Z'
REALTIME=$(date -u +"$TFMT" )
YYYY=$(date -u +%Y)
DDD=$(date -u +%j)
LOGFILE="$HOME/log/$APPNAME-$YYYY-$DDD.log"
ERRLOG="$HOME/log/$APPNAME-$YYYY-$DDD.errlog"
TMPDIR="$HOME/tmp/$APPNAME"
mkdir -p $TMPDIR
#
# User-adjustable items
SIZE="1024,768"
PWRMAX=90
SNRMAX=50
OXMAX=50
#
RIQARGS="-i -S"

declare -i OBSTIME CLEAN UOR MASK SMASK
# Process command line switches
CLEAN=$((0))
NETCDF=$((0))
SMASK=$((2#11111111))

while getopts 'chm:o:R:t:U:' OPTION
do
   case $OPTION in
      c) CLEAN=$((1)) 
         ;;
      f) FREQ="$OPTARG"
         ;;
      h)
         echo "powerplot-rti Version 2.05"
	 echo "Plot VIPIR ionograms in total power, SNR and O/X SNR"
         echo "Optional switches"
	 echo "-c Clean run by deleting the gnuplot data file before and after"
	 echo "   also clobbers the NetCDF file"
         echo "-f FREQ  set the displayed frequency"
         echo "-h Print this help message"
	 echo "-m MASK  Set the receiver mask to MASK"
         echo "-o OUTDIR  Set output directory, otherwise same as INFILE"
         echo "-R RANGE0  Manually set the range calibration"
         echo "-t {title} Add to the title of the plot"
         echo "-U URSI   URSI Code over-ride"
	 echo ""
	 exit 0 
	 ;;
      m) MASK=$(($OPTARG))
         ;;
      n) NOISE="$OPTARG"
         ;;
      o) OUTDIR="$OPTARG"
         ;;
      R) RANGE0="-R0 $OPTARG"
         ;;
      t) TITLE="$OPTARG"
         ;;
      U) URSI="$OPTARG"
         UOR=$((1))
         ;;
   esac
done
#
# Remove the switches from the list.  Remainder is a list of files to process
shift $((OPTIND - 1))
#   
#
FILELIST="$@"
for INFILE in $FILELIST ; do 
   REALTIME=$(date -u +"$TFMT" )
   if [ -z $INFILE ] ; then 
      echo "$REALTIME - $APPNAME - FATAL - Input file is undefined" >> $LOGFILE 
      exit -1
   fi
   if [ ! -r $INFILE ] ; then 
      echo "$REALTIME - $APPNAME - FATAL - Input file $INFILE does not exist or is non-readable" >> $LOGFILE 
      exit -2
   fi
#
   if [ -z $OUTDIR ] ; then
      DIR=$(dirname $INFILE)
   else
      DIR=$OUTDIR
   fi
#
#  Determine if the data file is compressed.
#  ZIP compression is not supported, but it could be.
#  need to preserve the ZIP file.
#
   BZIP2=$(file $INFILE | grep bzip2)
   GZIP=$(file $INFILE | grep gzip)
#
#  Determine file names
   if [ -n "$BZIP2" ] ; then 
         RIQFILE="$(dirname $INFILE)/$(basename $INFILE .bz2)" 
   elif [ -n "$GZIP" ] ; then
      RIQFILE="$(dirname $INFILE)/$(basename $INFILE .gz)"
   else
      RIQFILE="$INFILE"
   fi 
   BFN=$(basename $RIQFILE .RIQ)_RTI
#  For v0 data files with no station ID in the filename
   if (( $UOR )) ; then
      URSI="$URSI"
   else 
      URSI=${BFN:0:5}
   fi 
#  Determine the measurement time from the standard filename
   YEAR=${BFN:6:4}
   DAY=${BFN:10:3}
   HH=${BFN:13:2}
   MM=${BFN:15:2}
   SS=${BFN:17:2}
#
   DATESTRING="01Jan$YEAR $DAY days yesterday $HH:$MM:$SS"
   let OBSTIME=$(date -u +%s --date="$DATESTRING") >> $ERRLOG  2>&1  
   EXS=$?
   if [ $EXS != 0 ] ; then 
      echo "$REALTIME - $APPNAME - ERROR - Invalid date $DATESTRING"  >> $LOGFILE
      continue   # to next file
   fi  
   DATESTAMP=$(date -u +'%d%b%y'  --date="$DATESTRING" ) >> $ERRLOG  2>&1
   OBSDATE=$(date -u +'%Y %j %T %Z'  --date="$DATESTRING" ) >> $ERRLOG  2>&1 
#
#  Output and temp files
   TXTFILE=$DIR/$BFN".dat"
   PWRIMG=$DIR/$BFN"_PWR.png"
   SNRIMG=$DIR/$BFN"_SNR.png"
   OIMG=$DIR/$BFN"_O.png"
   XIMG=$DIR/$BFN"_X.png"
   OXIMG=$DIR/$BFN"_OX.png"
#
# 
# Optionally remove the text and netcdf files
   if (( $CLEAN )) ; then   
      echo "$REALTIME - $APPNAME - INFO - removing files $TXTFILE"  >> $LOGFILE
      rm -f $TXTFILE
   fi
#   
# decompress as needed
   if [ -n "$BZIP2" ] ; then
      echo "$REALTIME - $APPNAME - INFO - bzip2 decompress $INFILE"  >> $LOGFILE
      bunzip2 -qkfd "$INFILE" >> $ERRLOG 2>&1 
      EXS=$? 
      if [ $EXS != 0 ] ; then 
         echo "$REALTIME - $APPNAME - ERROR - bunzip2 failed with error $EXS"  >> $LOGFILE
         continue # to next file
      fi  
   elif [ -n "$GZIP" ] ; then
      echo "$REALTIME - $APPNAME - INFO - gzip decompress $INFILE" >> $LOGFILE
#     gzip does not allow to keep the compressed file. Force this.
      cp -a "$INFILE" $TMPDIR/ 
      gunzip -qfd "$INFILE" >> $ERRLOG 2>&1
      if [ $EXS != 0 ] ; then 
         echo "$REALTIME - $APPNAME - ERROR - gunzip failed with error $EXS"  >> $LOGFILE
         continue # to next file 
      fi  
      mv "$TMPDIR/$(basename $INFILE)" "$(dirname $INFILE)"
   fi
#
   REALTIME=$(date -u +"$TFMT" )
   echo "$REALTIME - $APPNAME - INFO - INFILE=$INFILE RIQFILE=$RIQFILE OUTDIR=$OUTDIR TITLE=$TITLE"  >> $LOGFILE 
#
#  Determine site-specific variables 
   source 'station_settings.sh' 
#
#  Over-ride the noise level by command line
   if [ -n "$NOISE" ] ; then 
      RXNOISE="-n $NOISE"
   fi
#  Over-ride the receiver mask by command line
   if [ $MASK ] ; then
      RXMASK="-m $MASK"
      BMASK=$( echo  "obase=2;$MASK" | bc)
   else
      RXMASK="-m $SMASK"
      BMASK=$( echo  "obase=2;$SMASK" | bc)
   fi
#
#  pad to 8 places
   BMASK=$(printf "%.8d\n" $BMASK)
#
   PLOTFILE=$TMPDIR/plot.gnu
#
#  Inform the calling program of the files created.
   echo $PWRIMG $SNRIMG $OXIMG 
#
   REALTIME=$(date -u +"$TFMT" )
   echo "$REALTIME - $APPNAME - INFO - processing data from $RIQFILE"  >> $LOGFILE

#  Use the readriq program to extract the plot file from the
#  raw data file.  Many user-maintained options here!
#  This is a long process; do not repeat if already done
#  ToDo: Some form of auto-selection based on format
   if [ ! -s "$TXTFILE"  ] ; then
#db      echo riq_ionogram $RIQARGS  $RANGE0 $RXMASK $RXNOISE $RIQFILE
      riq_ionogram  $RIQARGS -ba $RANGE0 $RXMASK $RXNOISE $RIQFILE > $TXTFILE 
   fi 
#
   FREQ=$(tail -2 < $TXTFILE | awk '{printf $5}')
  
#  Generate the gnuplot scripts
#
   REALTIME=$(date -u +"$TFMT" )
   echo "$REALTIME - $APPNAME - INFO - plotting data from $TXTFILE"  >> $LOGFILE

   cat <<EOF > $PLOTFILE
set pm3d map corners2color c1
set grid lt 22 lw 1
set xtics 2
set mxtics 2
set ytics 100
set mytics 10 
set yrange [00.0:200.0] 
set title "$STATION $OBSDATE       $DATESTAMP  $TITLE $FREQ MHz" font "arial,14"
set xlabel "Time [s]" font "arial,14"
set format x "%.1f"
set xtics font "arial,14"
set ylabel "Range [km]" font "arial,14"
set ytics font "arial,14"
set size 1.0, 1.0
set label 1  "VIPIR  $BFN.RIQ" at screen 0.65,0.11 font "arial,10"   
set label 3  "RxMask $BMASK" at screen 0.11,0.11 font "arial,10"
set term png font arial 12 size $SIZE  crop 
#
set cbrange [0:$PWRMAX]
set output "$PWRIMG"
splot "$TXTFILE" index 1 using 4:6:7  title "Total Power [dB]"
#
set output "$SNRIMG"
set cbrange [0:$SNRMAX] 
splot "$TXTFILE" using 4:6:8 title "SNR [dB]"
#
set output "$OXIMG"
set size 1.0, 1.0
set origin 0.0, 0.1
set multiplot
#set palette model HSV defined ( 0 0 1 1, 1 1 1 1 )
#set palette model HSV defined ( 0 0 1 0, 1 0 1 1, 6 0.8333 1 1, 7 0.8333 0 1)
#set palette model RGB defined ( -50 'green',-5 'black', +3 'black', 50 'red')  
set palette maxcolors 100 
#set palette model HSV functions 0.0, 1.0, gray
set palette model HSV functions 0.0, 1.0, gray
#set colorbox user horizontal origin 0.1, 0.12 size 0.35, 0.02 border
set colorbox user vertical origin 0.87, 0.30 size 0.02, 0.60 border
#set cblabel "O"
set label 2 "O" at screen 0.87,0.92 font "arial,14" 
set cbrange [0:$OXMAX]
set zrange [*:*]
set label 1 at screen 0.65,0.18
set label 3 at screen 0.11,0.18
#set label 3 "O&X SNR" at screen 0.87,0.95 font "arial,14"
splot "$TXTFILE" using 4:6:10  title "O&X SNR [dB]"
set zrange [3:*]
set palette model HSV functions 0.31, 1.0, gray
#set colorbox user horizontal origin 0.50, 0.12 size 0.35, 0.02 border
set colorbox user vertical origin 0.933, 0.30 size 0.02, 0.60 border
set cblabel "X"
set label 2 "X" at screen 0.94,0.92 font "arial,14" 
#Next line varies with gnuplot version?
#set title ""
splot "$TXTFILE" using 4:6:(\$11-\$9):12 title "" 
unset multiplot 

set output
exit

EOF

   gnuplot $PLOTFILE
   rm -f $PLOTFILE
#
#  Optionally remove the text file
   if (( $CLEAN )) ; then   
      echo "$REALTIME - $APPNAME - INFO - removing text file $TXTFILE"  >> $LOGFILE
      rm -f $TXTFILE
   fi

#  Remove original files if decompression was desired
   if [ -n "$BZIP2" ] || [ -n "$GZIP" ] ; then   
      echo "$REALTIME - $APPNAME - INFO - removing decompressed file $RIQFILE"  >> $LOGFILE
      rm -f $RIQFILE
   fi

   REALTIME=$(date -u +"$TFMT" )
   echo "$REALTIME - $APPNAME - INFO - Finished processing $INFILE"  >> $LOGFILE

done   #  each file

exit 0
