#!/bin/bash 
#
# spectrumplot.sh -- Generate Spectrum Analyzer plots
# for Scion HF radar raw data files. 
# Terry Bullett
# University of Colorado
#
# Version 2.05  29Sep10
# useage:
# powerplot-rti2.sh {switches} RIQFILES
#
# Requires:
#    msttcorefonts (included)
#    bc
#    riq-ionogram version 2.05+
#    station_settings.sh
# 
# Version 2.05 -- Moved station-based settings into a separate (sourced) function.
#
export GDFONTPATH=/usr/share/fonts/truetype/msttcorefonts/
#export GDFONTPATH=$HOME/lib/msttcorefonts/
#
APPNAME=spectrumplot
TFMT='%F %T %Z'
REALTIME=$(date -u +"$TFMT" )
YYYY=$(date -u +%Y)
DDD=$(date -u +%j)
LOGFILE="$HOME/log/$APPNAME-$YYYY-$DDD.log"
ERRLOG="$HOME/log/$APPNAME-$YYYY-$DDD.errlog"
TMPDIR="$HOME/tmp/$APPNAME"
mkdir -p $TMPDIR
#
# User-adustable plot parameters
SIZE="1024,768"
FRQSPAN="[0.0:25.0]"
PWRMAX=90
SNRMAX=50
OXMAX=50
#
RIQARGS='-i -sa -r1 30' 
declare -i OBSTIME CLEAN PEAK EIGHT MASK SMASK
# Process command line switches
CLEAN=$((0))
NETCDF=$((0))
EIGHT=$((0))
PEAK=$((0))
SMASK=$((2#11111111))

while getopts 'a:f: 
chp8' OPTION
do
   case $OPTION in
      c) CLEAN=$((1)) 
         ;;
      f) FRQSPAN="$OPTARG"
         ;;
      h)
         echo "spectrumplot Version 2.05"
	 echo "Plot VIPIR data in spectrum analyzer mode"
         echo "Optional switches"
	 echo "-a n  Plot antenna n  (UNIMPLEMENTED, use rxmask)"
         echo "-f FRQSPAN Change the frequency span"
	 echo "-c Clean up by deleting the temp data file"
         echo "-h Print this help message"
         echo "-m rxmask   Use rxmask for riq-ionogram"
         echo "-p Plot the Peak estimate.  Default is the noise (UNIMPLEMENTED)"
         echo "-t {title} Add to the title of the plot"
         echo "-8 Plot all 8 antennas in separate plots"
	 echo ""
	 exit 0 
	 ;;
      m) MASK=$(($OPTARG))
         ;;
      p) PEAK=$((1))
         ;;
      t) TITLE="$OPTARG"
         ;;
      8) EIGHT=$((1))
         ;;
   esac
done
#
# Remove the switches from the list.
shift $((OPTIND - 1))  
#
FILELIST="$@"
for INFILE in $FILELIST ; do 
   REALTIME=$(date -u +"$TFMT" )
   if [ -z $INFILE ] ; then 
      echo "$REALTIME - $APPNAME - FATAL - Input file is undefined" >> $LOGFILE 
      exit -1
   fi
   if [ ! -r $INFILE ] ; then 
      echo "$REALTIME - $APPNAME - FATAL - Input file $INFILE does not exist or is non-readable" >> $LOGFILE 
      exit -2
   fi
#
   if [ -z $OUTDIR ] ; then
      DIR=$(dirname $INFILE)
   else
      DIR=$OUTDIR
   fi
#
#  Determine if the data file is compressed.
#
#
#  Determine if the data file is compressed.
#  ZIP compression is not supported, but it could be.
#  need to preserve the ZIP file.
#
   BZIP2=$(file $INFILE | grep bzip2)
   GZIP=$(file $INFILE | grep gzip)
#
#  Determine file names
   if [ -n "$BZIP2" ] ; then 
         RIQFILE="$(dirname $INFILE)/$(basename $INFILE .bz2)" 
   elif [ -n "$GZIP" ] ; then
      RIQFILE="$(dirname $INFILE)/$(basename $INFILE .gz)"
   else
      RIQFILE="$INFILE"
   fi 
   BFN=$(basename $RIQFILE .RIQ)
#  For v0 data files with no station ID in the filename
   if (( $UOR )) ; then
      URSI="$URSI"
   else 
      URSI=${BFN:0:5}
   fi 

#  Determine the measurement time from the standard filename
   YEAR=${BFN:6:4}
   DAY=${BFN:10:3}
   HH=${BFN:13:2}
   MM=${BFN:15:2}
   SS=${BFN:17:2}
#
   DATESTRING="01Jan$YEAR $DAY days yesterday $HH:$MM:$SS"
   let OBSTIME=$(date -u +%s --date="$DATESTRING") >> $ERRLOG  2>&1  
   EXS=$?
   if [ $EXS != 0 ] ; then 
      echo "$REALTIME - $APPNAME - ERROR - Invalid date $DATESTRING"  >> $LOGFILE
      continue   # to next file
   fi  
   DATESTAMP=$(date -u +'%d%b%y'  --date="$DATESTRING" ) >> $ERRLOG  2>&1
   OBSDATE=$(date -u +'%Y %j %T %Z'  --date="$DATESTRING" ) >> $ERRLOG  2>&1 
#
#  Output and temp files

   REALTIME=$(date -u +"$TFMT" )
   echo "$REALTIME - $APPNAME - INFO - INFILE=$INFILE OUTDIR=$DIR TITLE=$TITLE"  >> $LOGFILE 
#
#  Determine site-specific variables 
   source 'station_settings.sh' 
#
#  Over-ride the receiver mask by command line
   if [ $MASK ] ; then
      RXMASK="-m $MASK"
      BMASK=$( echo  "obase=2;$MASK" | bc)
   else
      RXMASK="-m $SMASK"
      BMASK=$( echo  "obase=2;$SMASK" | bc)
   fi
#
#  pad to 8 places
   BMASK=$(printf "%.8d\n" $BMASK)
#
   if (( $EIGHT )) ; then 
      RXLIST='1 2 3 4 5 6 7 8'
      for RX in $RXLIST ; do
         TXTFILE="$DIR/$BFN-$RX"'.sam'
         PWRIMG="$DIR/$BFN-$RX"'_sam.png'
         PLOTFILE=$TMPDIR/plot.gnu
         rm -f $PLOTFILE
         if (( $CLEAN )) ; then 
            rm -f $TXTFILE 
         fi
#        Inform the calling program of the files created.
         echo $TXTFILE $PWRIMG 
#        Use the readriq program to extract the plot file from the
#        raw data file.  Many user-maintained options here!
#        This is a long process; do not repeat if already done
         if [ ! -s $TXTFILE  ] ; then 
            if (( $UOR )) ; then 
#              riq_ionogram  -c -bf -U $URSI $RANGE0 $RXMASK $RXNOISE $NCDFLAG $RIQFILE > $TXTFILE
#              Old Wallops  data prior to 2009
               riq_ionogram-0 $RIQARGS -bf -U $URSI $RXMASK $RIQFILE > $TXTFILE
            else
#               echo riq_ionogram  -c -bf  $RANGE0 $RXMASK $RXNOISE $NCDFLAG $RIQFILE
                riq_ionogram  $RIQARGS -bf $RANGE0 $RXMASK $RXNOISE $NCDFLAG $RIQFILE > $TXTFILE
            fi 
         fi  
      REALTIME=$(date -u +"$TFMT" )
      echo "$REALTIME - $APPNAME - INFO - plotting data from $TXTFILE"  >> $LOGFILE
      cat <<EOF > $PLOTFILE
set grid
set xrange $FRQSPAN
set xtics 2
set mxtics 5  
set yrange [0:$PWRMAX] 
set ytics 10
set mytics 10   
set title "$BFN $TITLE Receiver $RX"
set xlabel "Frequency [MHz]"
set ylabel "Power [dB]"
set label 3  "RxMask $BMASK" at screen 0.80,0.95 font "arial,10"
set term png font arial 12 size 1024,768 crop 
set output "$PWRIMG"
plot "$TXTFILE" using 2:6 title "Noise" with fsteps lt 1 , "$TXTFILE" using 2:7 title "Peak" with fsteps lt 2
set output
exit
EOF
         gnuplot $PLOTFILE
         if (( $CLEAN )) ; then 
            rm -f $TXTFILE 
         fi

#         rm -f $PLOTFILE
      done  # each receiver
   else
#     Standard power mode for all 8 receivers summed incoherently
      TXTFILE="$DIR/$BFN"'.sam'
      PWRIMG="$DIR/$BFN"'_sam.png'
      PLOTFILE=$TMPDIR/plot.gnu
      rm -f $PLOTFILE 
      if (( $CLEAN )) ; then 
         rm -f $TXTFILE 
      fi
#     Inform the calling program of the files created.
      echo $TXTFILE $PWRIMG 
#     Use the readriq program to extract the plot file from the
#     raw data file.  Many user-maintained options here!
#     This is a long process; do not repeat if already done
      if [ ! -s $TXTFILE  ] ; then 
         if (( $UOR )) ; then 
#           riq_ionogram  $RIQARGS -U $URSI $RANGE0 $RXMASK $RXNOISE $NCDFLAG $RIQFILE > $TXTFILE
#           Old Wallops  data prior to 2009
            riq_ionogram-0 $RIQARGS -U $URSI $RXMASK $RIQFILE > $TXTFILE
         else
#             echo riq_ionogram  $RIQARGS  $RANGE0 $RXMASK $RXNOISE $NCDFLAG $RIQFILE
             riq_ionogram  $RIQARGS $RANGE0 $RXMASK $RXNOISE $NCDFLAG $RIQFILE > $TXTFILE
         fi 
      fi  
      REALTIME=$(date -u +"$TFMT" )
      echo "$REALTIME - $APPNAME - INFO - plotting data from $TXTFILE"  >> $LOGFILE
      cat <<EOF > $PLOTFILE
set grid 
set xrange $FRQSPAN
set xtics 2
set mxtics 5  
set yrange [0:$PWRMAX]
set ytics 10
set mytics 10   
set title "$BFN $TITLE"
set xlabel "Frequency [MHz]"
set ylabel 'Power [dB]'
set term png font arial 12 size 1024,768 crop 
set output "$PWRIMG"
set label 1  "VIPIR  $BFN.RIQ" at screen 0.70,0.02 font "arial,10"   
set label 3  "RxMask $BMASK" at screen 0.11,0.02 font "arial,10"
plot "$TXTFILE" using 2:6 title "Noise" with fsteps lt 1 , "$TXTFILE" using 2:7 title "Peak" with fsteps lt 2
set output
exit
EOF
      gnuplot $PLOTFILE
      if (( $CLEAN )) ; then 
         rm -f $TXTFILE 
      fi

#   rm -f $PLOTFILE
   fi
done   #  Each input file 

exit 0
